@@ -306,6 +306,73 @@ def parsefile():
 ##     </root>
 ##     """
 
+def iterparse():
+    """
+    Test iterparse interface.
+
+    >>> iterparse = ElementTree.iterparse
+
+    >>> context = iterparse("samples/simple.xml")
+    >>> for action, elem in context:
+    ...   print action, elem.tag
+    end element
+    end element
+    end empty-element
+    end root
+    >>> context.root.tag
+    'root'
+
+    >>> context = iterparse("samples/simple-ns.xml")
+    >>> for action, elem in context:
+    ...   print action, elem.tag
+    end {namespace}element
+    end {namespace}element
+    end {namespace}empty-element
+    end {namespace}root
+
+    >>> events = ()
+    >>> context = iterparse("samples/simple.xml", events)
+    >>> for action, elem in context:
+    ...   print action, elem.tag
+
+    >>> events = ()
+    >>> context = iterparse("samples/simple.xml", events=events)
+    >>> for action, elem in context:
+    ...   print action, elem.tag
+
+    >>> events = ("start", "end")
+    >>> context = iterparse("samples/simple.xml", events)
+    >>> for action, elem in context:
+    ...   print action, elem.tag
+    start root
+    start element
+    end element
+    start element
+    end element
+    start empty-element
+    end empty-element
+    end root
+
+    >>> events = ("start", "end", "start-ns", "end-ns")
+    >>> context = iterparse("samples/simple-ns.xml", events)
+    >>> for action, elem in context:
+    ...   if action in ("start", "end"):
+    ...     print action, elem.tag
+    ...   else:
+    ...     print action, elem
+    start-ns ('', 'namespace')
+    start {namespace}root
+    start {namespace}element
+    end {namespace}element
+    start {namespace}element
+    end {namespace}element
+    start {namespace}empty-element
+    end {namespace}empty-element
+    end {namespace}root
+    end-ns None
+
+    """
+
 ## def fancyparsefile():
 ##     """
 ##     Test the "fancy" parser.
