@@ -442,6 +442,7 @@ class ETreeTestCaseBase(unittest.TestCase):
         ElementTree = self.etree.ElementTree
         XML = self.etree.XML
         Comment = self.etree.Comment
+        ProcessingInstruction = self.etree.ProcessingInstruction
         
         el = Element('hoi')
         self.assert_(iselement(el))
@@ -455,6 +456,9 @@ class ETreeTestCaseBase(unittest.TestCase):
 
         c = Comment('test')
         self.assert_(iselement(c))
+
+        p = ProcessingInstruction("test", "some text")
+        self.assert_(iselement(p))
         
     def test_iteration(self):
         XML = self.etree.XML
@@ -707,6 +711,16 @@ class ETreeTestCaseBase(unittest.TestCase):
             a.text)
         self.assertXML('<a></a>', a)
         
+    def test_set_text_empty(self):
+        Element = self.etree.Element
+
+        a = Element('a')
+        self.assertEquals(None, a.text)
+
+        a.text = ''
+        self.assertEquals('', a.text)
+        self.assertXML('<a></a>', a)
+        
     def test_tail1(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -753,7 +767,7 @@ class ETreeTestCaseBase(unittest.TestCase):
             None,
             a.tail)
         self.assertXML('<a></a>', a)
-        
+
     def test_comment(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -761,6 +775,7 @@ class ETreeTestCaseBase(unittest.TestCase):
 
         a = Element('a')
         a.append(Comment('foo'))
+        self.assertEquals(a[0].tag, Comment)
         self.assertEquals(a[0].text, 'foo')
 
     def test_comment_text(self):
@@ -796,6 +811,30 @@ class ETreeTestCaseBase(unittest.TestCase):
         for i in c:
             pass
 
+    def test_pi(self):
+        # lxml.etree separates target and text
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        ProcessingInstruction = self.etree.ProcessingInstruction
+
+        a = Element('a')
+        a.append(ProcessingInstruction('foo', 'some more text'))
+        self.assertEquals(a[0].tag, ProcessingInstruction)
+        self.assertXML("<a><?foo some more text?></a>",
+                       a)
+
+    def test_pi_nonsense(self):
+        ProcessingInstruction = self.etree.ProcessingInstruction
+        pi = ProcessingInstruction('foo')
+        self.assertEquals({}, pi.attrib)
+        self.assertEquals([], pi.keys())
+        self.assertEquals([], pi.items())
+        self.assertEquals(None, pi.get('hoi'))
+        self.assertEquals(0, len(pi))
+        # should not iterate
+        for i in pi:
+            pass
+
     def test_setitem(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -1796,7 +1835,114 @@ class ETreeTestCaseBase(unittest.TestCase):
 
         self.assert_(tostring(b) == '<b/>Foo' or
                      tostring(b) == '<b />Foo')
-        
+
+    def test_iterparse(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b></b><c/></a>')
+
+        iterator = iterparse(f)
+        self.assertEquals(None,
+                          iterator.root)
+        events = list(iterator)
+        root = iterator.root
+        self.assertEquals(
+            [('end', root[0]), ('end', root[1]), ('end', root)],
+            events)
+
+    def test_iterparse_file(self):
+        iterparse = self.etree.iterparse
+        iterator = iterparse(fileInTestDir("test.xml"))
+        self.assertEquals(None,
+                          iterator.root)
+        events = list(iterator)
+        root = iterator.root
+        self.assertEquals(
+            [('end', root[0]), ('end', root)],
+            events)
+
+    def test_iterparse_start(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b></b><c/></a>')
+
+        iterator = iterparse(f, events=('start',))
+        events = list(iterator)
+        root = iterator.root
+        self.assertEquals(
+            [('start', root), ('start', root[0]), ('start', root[1])],
+            events)
+
+    def test_iterparse_start_end(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b></b><c/></a>')
+
+        iterator = iterparse(f, events=('start','end'))
+        events = list(iterator)
+        root = iterator.root
+        self.assertEquals(
+            [('start', root), ('start', root[0]), ('end', root[0]),
+             ('start', root[1]), ('end', root[1]), ('end', root)],
+            events)
+
+    def test_iterparse_clear(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b></b><c/></a>')
+
+        iterator = iterparse(f)
+        for event, elem in iterator:
+            elem.clear()
+
+        root = iterator.root
+        self.assertEquals(0,
+                          len(root))
+
+    def test_iterparse_large(self):
+        iterparse = self.etree.iterparse
+        CHILD_COUNT = 12345
+        f = StringIO('<a>' + ('<b>test</b>'*CHILD_COUNT) + '</a>')
+
+        i = 0
+        for key in iterparse(f):
+            event, element = key
+            i += 1
+        self.assertEquals(i, CHILD_COUNT + 1)
+
+    def test_iterparse_attrib_ns(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a xmlns="ns1"><b><c xmlns="ns2"/></b></a>')
+
+        attr_name = '{testns}bla'
+        events = []
+        iterator = iterparse(f, events=('start','end','start-ns','end-ns'))
+        for event, elem in iterator:
+            events.append(event)
+            if event == 'start':
+                if elem.tag != '{ns1}a':
+                    elem.set(attr_name, 'value')
+
+        self.assertEquals(
+            ['start-ns', 'start', 'start', 'start-ns', 'start',
+             'end', 'end-ns', 'end', 'end', 'end-ns'],
+            events)
+
+        root = iterator.root
+        self.assertEquals(
+            None,
+            root.get(attr_name))
+        self.assertEquals(
+            'value',
+            root[0].get(attr_name))
+
+    def test_iterparse_getiterator(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b><d/></b><c/></a>')
+
+        counts = []
+        for event, elem in iterparse(f):
+            counts.append(len(list(elem.getiterator())))
+        self.assertEquals(
+            [1,2,1,4],
+            counts)
+
     def test_parse_file(self):
         parse = self.etree.parse
         # from file
