@@ -28,6 +28,9 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assert_(etree.__version__.startswith(
             str(etree.LXML_VERSION[0])))
 
+    def test_c_api(self):
+        self.assert_(hasattr(self.etree, '_import_c_api'))
+
     def test_element_names(self):
         Element = self.etree.Element
         
@@ -62,7 +65,35 @@ class ETreeOnlyTestCase(HelperTestCase):
             "valueB",
             root[0].get("default"))
 
+    def test_pi(self):
+        # lxml.etree separates target and text
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        ProcessingInstruction = self.etree.ProcessingInstruction
+
+        a = Element('a')
+        a.append(ProcessingInstruction('foo', 'some more text'))
+        self.assertEquals(a[0].target, 'foo')
+        self.assertEquals(a[0].text, 'some more text')
+
+    def test_pi_parse(self):
+        XML = self.etree.XML
+        root = XML("<test><?mypi my test ?></test>")
+        self.assertEquals(root[0].target, "mypi")
+        self.assertEquals(root[0].text, "my test ")
+
+    def test_attribute_set(self):
+        # ElementTree accepts arbitrary attribute values
+        # lxml.etree allows only strings
+        Element = self.etree.Element
+
+        root = Element("root")
+        root.set("attr", "TEST")
+        self.assertEquals("TEST", root.get("attr"))
+        self.assertRaises(TypeError, root.set, "newattr", 5)
+
     def test_parse_error(self):
+        # ET raises ExpatError
         parse = self.etree.parse
         # from StringIO
         f = StringIO('<a><b></c></b></a>')
@@ -70,6 +101,7 @@ class ETreeOnlyTestCase(HelperTestCase):
         f.close()
 
     def test_parse_parser_type_error(self):
+        # ET raises IOError only
         parse = self.etree.parse
         self.assertRaises(TypeError, parse, 'notthere.xml', object())
 
@@ -98,6 +130,153 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertRaises(SyntaxError, parse, f)
         f.close()
 
+    def test_iterparse_broken(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b><c/></a>')
+        # ET raises ExpatError, lxml raises XMLSyntaxError
+        self.assertRaises(self.etree.XMLSyntaxError, list, iterparse(f))
+
+    def test_iterparse_file_dtd(self):
+        iterparse = self.etree.iterparse
+        iterator = iterparse(fileInTestDir("test.xml"), events=("start",),
+                             attribute_defaults=True)
+        attributes = [ element.get("default")
+                       for event, element in iterator ]
+        self.assertEquals(
+            ["valueA", "valueB"],
+            attributes)
+
+    def test_iterparse_strip(self):
+        iterparse = self.etree.iterparse
+        f = StringIO("""
+               <a>  \n \n  <b> b test </b>  \n
+
+               \n\t <c> \n </c> </a>  \n """)
+        iterator = iterparse(f, remove_blank_text=True)
+        text = [ (element.text, element.tail)
+                 for event, element in iterator ]
+        self.assertEquals(
+            [(" b test ", None), (" \n ", None), (None, None)],
+            text)
+
+    def test_iterparse_tag(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b><d/></b><c/></a>')
+
+        iterator = iterparse(f, tag="b", events=('start', 'end'))
+        events = list(iterator)
+        root = iterator.root
+        self.assertEquals(
+            [('start', root[0]), ('end', root[0])],
+            events)
+
+    def test_iterparse_tag_all(self):
+        iterparse = self.etree.iterparse
+        f = StringIO('<a><b><d/></b><c/></a>')
+
+        iterator = iterparse(f, tag="*", events=('start', 'end'))
+        events = list(iterator)
+        self.assertEquals(
+            8,
+            len(events))
+
+    def test_iterwalk_tag(self):
+        iterwalk = self.etree.iterwalk
+        root = self.etree.XML('<a><b><d/></b><c/></a>')
+
+        iterator = iterwalk(root, tag="b", events=('start', 'end'))
+        events = list(iterator)
+        self.assertEquals(
+            [('start', root[0]), ('end', root[0])],
+            events)
+
+    def test_iterwalk_tag_all(self):
+        iterwalk = self.etree.iterwalk
+        root = self.etree.XML('<a><b><d/></b><c/></a>')
+
+        iterator = iterwalk(root, tag="*", events=('start', 'end'))
+        events = list(iterator)
+        self.assertEquals(
+            8,
+            len(events))
+
+    def test_iterwalk(self):
+        iterwalk = self.etree.iterwalk
+        root = self.etree.XML('<a><b></b><c/></a>')
+
+        events = list(iterwalk(root))
+        self.assertEquals(
+            [('end', root[0]), ('end', root[1]), ('end', root)],
+            events)
+
+    def test_iterwalk_start(self):
+        iterwalk = self.etree.iterwalk
+        root = self.etree.XML('<a><b></b><c/></a>')
+
+        iterator = iterwalk(root, events=('start',))
+        events = list(iterator)
+        self.assertEquals(
+            [('start', root), ('start', root[0]), ('start', root[1])],
+            events)
+
+    def test_iterwalk_start_end(self):
+        iterwalk = self.etree.iterwalk
+        root = self.etree.XML('<a><b></b><c/></a>')
+
+        iterator = iterwalk(root, events=('start','end'))
+        events = list(iterator)
+        self.assertEquals(
+            [('start', root), ('start', root[0]), ('end', root[0]),
+             ('start', root[1]), ('end', root[1]), ('end', root)],
+            events)
+
+    def test_iterwalk_clear(self):
+        iterwalk = self.etree.iterwalk
+        root = self.etree.XML('<a><b></b><c/></a>')
+
+        iterator = iterwalk(root)
+        for event, elem in iterator:
+            elem.clear()
+
+        self.assertEquals(0,
+                          len(root))
+
+    def test_iterwalk_attrib_ns(self):
+        iterwalk = self.etree.iterwalk
+        root = self.etree.XML('<a xmlns="ns1"><b><c xmlns="ns2"/></b></a>')
+
+        attr_name = '{testns}bla'
+        events = []
+        iterator = iterwalk(root, events=('start','end','start-ns','end-ns'))
+        for event, elem in iterator:
+            events.append(event)
+            if event == 'start':
+                if elem.tag != '{ns1}a':
+                    elem.set(attr_name, 'value')
+
+        self.assertEquals(
+            ['start-ns', 'start', 'start', 'start-ns', 'start',
+             'end', 'end-ns', 'end', 'end', 'end-ns'],
+            events)
+
+        self.assertEquals(
+            None,
+            root.get(attr_name))
+        self.assertEquals(
+            'value',
+            root[0].get(attr_name))
+
+    def test_iterwalk_getiterator(self):
+        iterwalk = self.etree.iterwalk
+        root = self.etree.XML('<a><b><d/></b><c/></a>')
+
+        counts = []
+        for event, elem in iterwalk(root):
+            counts.append(len(list(elem.getiterator())))
+        self.assertEquals(
+            [1,2,1,4],
+            counts)
+
     def test_resolve_string_dtd(self):
         parse = self.etree.parse
         parser = self.etree.XMLParser(dtd_validation=True)
@@ -241,6 +420,18 @@ class ETreeOnlyTestCase(HelperTestCase):
             '<a><!-- foo  --></a>',
             tostring(a))
 
+    # does not raise an exception in ElementTree
+    def test_comment_immutable(self):
+        Element = self.etree.Element
+        Comment = self.etree.Comment
+
+        c = Comment()
+        el = Element('myel')
+
+        self.assertRaises(TypeError, c.append, el)
+        self.assertRaises(TypeError, c.insert, 0, el)
+        self.assertRaises(TypeError, c.set, "myattr", "test")
+
     # test weird dictionary interaction leading to segfault previously
     def test_weird_dict_interaction(self):
         root = self.etree.Element('root')
@@ -298,6 +489,113 @@ class ETreeOnlyTestCase(HelperTestCase):
             b,
             d.getparent())
 
+    def test_iterchildren(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc><one/><two>Two</two>Hm<three/></doc>')
+        result = []
+        for el in root.iterchildren():
+            result.append(el.tag)
+        self.assertEquals(['one', 'two', 'three'], result)
+
+    def test_iterchildren_reversed(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc><one/><two>Two</two>Hm<three/></doc>')
+        result = []
+        for el in root.iterchildren(reversed=True):
+            result.append(el.tag)
+        self.assertEquals(['three', 'two', 'one'], result)
+
+    def test_iterchildren_tag(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc><one/><two>Two</two>Hm<two>Bla</two></doc>')
+        result = []
+        for el in root.iterchildren(tag='two'):
+            result.append(el.text)
+        self.assertEquals(['Two', 'Bla'], result)
+
+    def test_iterchildren_tag_reversed(self):
+        XML = self.etree.XML
+        
+        root = XML('<doc><one/><two>Two</two>Hm<two>Bla</two></doc>')
+        result = []
+        for el in root.iterchildren(reversed=True, tag='two'):
+            result.append(el.text)
+        self.assertEquals(['Bla', 'Two'], result)
+
+    def test_iterancestors(self):
+        Element    = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(b, 'd')
+        self.assertEquals(
+            [],
+            list(a.iterancestors()))
+        self.assertEquals(
+            [a],
+            list(b.iterancestors()))
+        self.assertEquals(
+            a,
+            c.iterancestors().next())
+        self.assertEquals(
+            [b, a],
+            list(d.iterancestors()))
+
+    def test_iterancestors_tag(self):
+        Element    = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(b, 'd')
+        self.assertEquals(
+            [a],
+            list(d.iterancestors(tag='a')))
+
+    def test_iterdescendants(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(b, 'd')
+        e = SubElement(c, 'e')
+
+        self.assertEquals(
+            [b, d, c, e],
+            list(a.iterdescendants()))
+        self.assertEquals(
+            [],
+            list(d.iterdescendants()))
+
+    def test_iterdescendants_tag(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(b, 'd')
+        e = SubElement(c, 'e')
+
+        self.assertEquals(
+            [],
+            list(a.iterdescendants('a')))
+        a2 = SubElement(e, 'a')
+        self.assertEquals(
+            [a2],
+            list(a.iterdescendants('a')))
+        self.assertEquals(
+            [a2],
+            list(c.iterdescendants('a')))
+
     def test_getroottree(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
@@ -316,6 +614,89 @@ class ETreeOnlyTestCase(HelperTestCase):
             a,
             d.getroottree().getroot())
 
+    def test_getnext(self):
+        Element    = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        self.assertEquals(
+            None,
+            a.getnext())
+        self.assertEquals(
+            c,
+            b.getnext())
+        self.assertEquals(
+            None,
+            c.getnext())
+
+    def test_getprevious(self):
+        Element    = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(b, 'd')
+        self.assertEquals(
+            None,
+            a.getprevious())
+        self.assertEquals(
+            b,
+            c.getprevious())
+        self.assertEquals(
+            None,
+            b.getprevious())
+
+    def test_itersiblings(self):
+        Element    = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(b, 'd')
+        self.assertEquals(
+            [],
+            list(a.itersiblings()))
+        self.assertEquals(
+            [c],
+            list(b.itersiblings()))
+        self.assertEquals(
+            c,
+            b.itersiblings().next())
+        self.assertEquals(
+            [],
+            list(c.itersiblings()))
+        self.assertEquals(
+            [b],
+            list(c.itersiblings(preceding=True)))
+        self.assertEquals(
+            [],
+            list(b.itersiblings(preceding=True)))
+
+    def test_itersiblings_tag(self):
+        Element    = self.etree.Element
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        c = SubElement(a, 'c')
+        d = SubElement(b, 'd')
+        self.assertEquals(
+            [],
+            list(a.itersiblings(tag='XXX')))
+        self.assertEquals(
+            [c],
+            list(b.itersiblings(tag='c')))
+        self.assertEquals(
+            [b],
+            list(c.itersiblings(preceding=True, tag='b')))
+        self.assertEquals(
+            [],
+            list(c.itersiblings(preceding=True, tag='c')))
+
     def test_parseid(self):
         parseid = self.etree.parseid
         XML     = self.etree.XML
@@ -470,6 +851,36 @@ class ETreeOnlyTestCase(HelperTestCase):
             '<z xmlns="http://ns.infrae.com/foo" xmlns:hoi="http://ns.infrae.com/hoi"><hoi:x></hoi:x></z>',
             self._writeElement(e))
 
+    def test_element_nsmap(self):
+        etree = self.etree
+
+        r = {None: 'http://ns.infrae.com/foo',
+             'hoi': 'http://ns.infrae.com/hoi'}
+        e = etree.Element('{http://ns.infrae.com/foo}bar', nsmap=r)
+        self.assertEquals(
+            r,
+            e.nsmap)
+
+    def test_subelement_nsmap(self):
+        etree = self.etree
+
+        re = {None: 'http://ns.infrae.com/foo',
+             'hoi': 'http://ns.infrae.com/hoi'}
+        e = etree.Element('{http://ns.infrae.com/foo}bar', nsmap=re)
+
+        rs = {None: 'http://ns.infrae.com/honk',
+             'top': 'http://ns.infrae.com/top'}
+        s = etree.SubElement(e, '{http://ns.infrae.com/honk}bar', nsmap=rs)
+
+        r = re.copy()
+        r.update(rs)
+        self.assertEquals(
+            re,
+            e.nsmap)
+        self.assertEquals(
+            r,
+            s.nsmap)
+
     def test_getiterator_filter_namespace(self):
         Element = self.etree.Element
         SubElement = self.etree.SubElement
